#!/usr/bin/env python
# -*- coding: utf-8 -*-

#WRITTEN BY DENE BOWDALO

###------------------------------------------------------------------------------------###
###IMPORT MODULES
###------------------------------------------------------------------------------------###

import __builtin__
import csv
import crossnetwork_processing_modules
import glob
from mpi4py import MPI
import numpy as np
import os
import Queue

###------------------------------------------------------------------------------------###
###------------------------------------------------------------------------------------###

WORKTAG = 1
DIETAG = 0

#LOAD GHOST CONFIGURATION VARIABLES
config_frame = np.load('/esarchive/obs/ghost/process/run/GHOST_configuration.npz')
run_details = config_frame['run_details'].item()
parameter_details = config_frame['parameter_details'].item()
network_details = config_frame['network_details'].item()

#MAKE RUN/PARAMETER/NETWORK DATA GLOBAL ACROSS ALL SUBMODULES
__builtin__.run_details = run_details
__builtin__.parameter_details = parameter_details
__builtin__.network_details = network_details

#as an exception load EIONET metadata all into memory as no way to make join between data and metadata without opening data files 
#for EIONET this means loading 3 files into memory: 
if run_details['network'] == 'EIONET':
    #read standard EIONET metadata file 

    external_metadata_file = '/esarchive/obs/ghost/%s/metadata/network_provided/EIONET_META.csv'%(run_details['network'])
        
    with open(external_metadata_file, 'rb') as f:
        reader = csv.reader(f,delimiter=',')
        for row_i, row in enumerate(reader):
            if row_i == 0:
                meta_variable_names = row
                external_metadata = [[] for _ in xrange(len(meta_variable_names))]
            else: 
                for meta_variable_name in meta_variable_names:
                    external_metadata[meta_variable_names.index(meta_variable_name)].append(row[meta_variable_names.index(meta_variable_name)].strip())
            
        #convert to numpy record array, to allow indexing with column names
        external_metadata = np.rec.fromarrays(external_metadata, names=meta_variable_names) 
        #make standard external metadata accesible globally
        __builtin__.external_metadata = external_metadata
    
    #read extra metadata AIDE D file
    EIONET_AIDE_D_metadata_file = '/esarchive/obs/ghost/%s/metadata/network_provided/EIONET_AIDE_D.csv'%(run_details['network'])
    with open(EIONET_AIDE_D_metadata_file, 'rb') as f:
        reader = csv.reader(f,delimiter=',')
        for row_i, row in enumerate(reader):
            if row_i == 0:
                meta_variable_names = row
                EIONET_AIDE_D_metadata = [[] for _ in xrange(len(meta_variable_names))]
            else: 
                for meta_variable_name in meta_variable_names:
                    EIONET_AIDE_D_metadata[meta_variable_names.index(meta_variable_name)].append(row[meta_variable_names.index(meta_variable_name)].strip())
                                    
    #convert to numpy record array, to allow indexing with column names
    EIONET_AIDE_D_metadata = np.rec.fromarrays(EIONET_AIDE_D_metadata, names=meta_variable_names)
    #make AIDE D metadata accesible globally
    __builtin__.external_metadata_AIDE_D = EIONET_AIDE_D_metadata
    
    #read AIRBASE v8 processed .npy file
    AIRBASE_v8_metadata_file = '/esarchive/obs/ghost/AIRBASE/metadata/processed/AIRBASE_META.npy'
    AIRBASE_v8_metadata = np.load(AIRBASE_v8_metadata_file) 
    #make AIRBASE v8 metadata accesible globally
    __builtin__.external_metadata_AIRBASE = AIRBASE_v8_metadata

#IMPORT SPECIFIC NETWORK PROCESSING MODULE FOR NETWORK BEING PROCESSED
network_processor = __import__(run_details['network']+'_processor')

#UPDATE GLOBAL WRITE STAGE
write_stage = 'first_write'
__builtin__.write_stage = write_stage

class Work(object):
    def __init__(self):
    
        #CREATE QUEUE FOR ADDING DATA TO
        self.work = Queue.Queue()
    
        #GET ALL DATA FILES FOR SPECIFIC PARAMETER, FOR NETWORK 
        data_files = crossnetwork_processing_modules.get_data_files()
    
        #MAKE SURE HAVE SOME DATA TO PROCESS
        if len(data_files) > 0:
    
            #TEST FOR PRESENCE OF TEMPORARY STATION DATA FILE DIRECTORY. USED TO STORE INITIAL NUMPY .npz FILES OF RAW DATA BEFORE QA CHECKS.
            #IF DOES NOT EXIST CREATE IT
            temp_file_dir = '/esarchive/obs/ghost/%s/native/%s/temporary/'%(run_details['network'],parameter_details['bsc_parameter_name'])
            does_temporary_exist = os.path.isdir(temp_file_dir)
            if does_temporary_exist == False:
                os.makedirs(temp_file_dir)
            #DELETE ALL CURRENT TEMPORARY STATION DATA FILES FOR PARAMETER/NETWORK (IN CASE OF FAILED PREVIOUS RUN AND HAVE NOT BEEN CLEANED)
            for zippath in glob.iglob(os.path.join(temp_file_dir, '*.npz')):
                os.remove(zippath)
    
            #GET ALL DIFFERENT YEARS OF FILENAMES DATA IS SEPARATED BETWEEN
            if network_details['file_storage_type'] == 'all_per_year':
                data_years = range(int(run_details['start_year']),int(run_details['end_year']))
            elif (network_details['file_storage_type'] == 'site_per_year') or (network_details['file_storage_type'] == 'site_per_multiyear'):
                data_years = range(1)
        
            #BRANCH HERE, DEPENDENT ON HOW OBSERVATIONS ARE REPORTED
            #1. IF FILE STORAGE TYPE IS ALL SITES PER YEAR, PROCESS ALL STATIONS YEAR BY YEAR
            #2. IF FILE STORAGE TYPE IS SITE PER YEAR/SITE PER MULTIYEAR, PROCESS EACH ALL YEARS STATION BY STATION
    
            #ITERATE THROUGH ALL YEARS OF DATA FILES (JUST 1 YEAR IF HAVE SITES REPORTED PER YEAR)
            for year in data_years:
            #IF FILE STORAGE TYPE IS ALL SITES PER YEAR, LIMIT DATA FILES TO RELEVANT YEARLY FILE
                if network_details['file_storage_type'] == 'all_per_year':
                    data_files_to_process = [s for s in data_files if str(year) in s]
                    print 'PROCESSING YEAR: %s'%(year)
                #OTHERWISE DO NOT FILTER FILES BY YEAR
                elif (network_details['file_storage_type'] == 'site_per_year') or (network_details['file_storage_type'] == 'site_per_multiyear'):
                    data_files_to_process = data_files
        
                #GET ALL STATION REFERENCES & REFERENCES TO CONNECT METADATA/DATA FROM DATA FILE/S
                station_references = crossnetwork_processing_modules.get_station_references(data_files_to_process) 
                #station_references = station_references[880:900]
                #HOW MANY STATIONS TO PROCESS?
                print 'NUMBER OF STATIONS: %s'%(len(station_references))
                
    
                #LOAD EXTERNAL METADATA INTO MEMORY (*IF EXISTS FOR NETWORK)        
                if network_details['external_metadata_file'] != '':
                    external_metadata = crossnetwork_processing_modules.read_external_metadata(station_references)
                else:
                    external_metadata = [[] for _ in xrange(len(station_references))]
    
                #IF FILE STORAGE TYPE IS SITE PER YEAR/SITE PER MULTIYEAR, SPLIT DATA FILES INTO LIST OF LISTS CONTAINING STATION SPECIFIC DATA FILES
                if (network_details['file_storage_type'] == 'site_per_year') or (network_details['file_storage_type'] == 'site_per_multiyear'):
                    data_files_to_process = crossnetwork_processing_modules.generate_data_files_LoL(station_references,data_files_to_process)

                #ZIP VARIABLES THAT WILL BE PROCESSED
                zipped_list = zip(range(len(station_references)),station_references,data_files_to_process,external_metadata)
    
                #ADD DATA TO QUEUE
                for data in zipped_list:
                    self.work.put(data)

    def get_next(self):
        if self.work.empty():
            return None
        return self.work.get()
        
    def test_if_start_empty(self):
        if self.work.empty():
            return True
        else:
            return False
 

def do_work(comm,work):
    process_code = network_processor.read_data(work)
    if process_code == 0:
        comm.Abort()
    return work
    
def master(comm):
    num_procs = comm.Get_size()
    status = MPI.Status()
    
    # generate work queue
    wq = Work()
    
    #array for appending results
    results = []
    
    #test if queue is empty to start with
    if wq.test_if_start_empty():
        # if empty - Tell all the slaves to exit by sending an empty message with DIETAG
        for rank in xrange(1, num_procs):
            comm.send(0, dest=rank, tag=DIETAG)
        return

    # Seed the slaves, send one unit of work to each slave (rank)
    working_ranks = []
    for rank in xrange(1, num_procs):
        work = wq.get_next()
        if not work: break
        comm.send(work, dest=rank, tag=WORKTAG)
        working_ranks.append(rank)
    
    # Loop over getting new work requests until there is no more work to be done
    while True:
        work = wq.get_next()
        if not work: break
    
        # Receive results from a slave
        results.append(comm.recv(source=MPI.ANY_SOURCE, tag=MPI.ANY_TAG, status=status))
        
        # Send the slave a new work unit
        comm.send(work, dest=status.Get_source(), tag=WORKTAG)
    
    # No more work to be done, receive all outstanding results from slaves
    for rank in working_ranks:
        results.append(comm.recv(source=MPI.ANY_SOURCE, tag=MPI.ANY_TAG, status=status))

    # Tell all the slaves to exit by sending an empty message with DIETAG
    for rank in xrange(1, num_procs):
        comm.send(0, dest=rank, tag=DIETAG)
        
    return results


def slave(comm):
    my_rank = comm.Get_rank()
    status = MPI.Status()

    while True:
        # Receive a message from the master (the work to do)
        work = comm.recv(source=0, tag=MPI.ANY_TAG, status=status)
        # Check the tag of the received message
        if status.Get_tag() == DIETAG: break 

        # Do the work and send it back
        result = do_work(comm,work)
        comm.send(result, dest=0)
        
def main():
    comm = MPI.COMM_WORLD
    my_rank = comm.Get_rank()
    my_name = MPI.Get_processor_name()
    
    if my_rank == 0:
        results = master(comm)
        if results is not None:
            print 'N1 DONE'
        else:
            print 'N1 NO DATA'
    else:
        slave(comm)


if __name__ == '__main__':
    main()