#!/usr/bin/env python
# -*- coding: utf-8 -*-

#WRITTEN BY DENE BOWDALO

###------------------------------------------------------------------------------------###
###IMPORT MODULES
###------------------------------------------------------------------------------------###

import __builtin__
import crossnetwork_processing_modules
import glob
from mpi4py import MPI
import numpy as np
import os
import Queue

###------------------------------------------------------------------------------------###
###------------------------------------------------------------------------------------###

WORKTAG = 1
DIETAG = 0

#LOAD GHOST CONFIGURATION VARIABLES
config_frame = np.load('/esarchive/obs/ghost/process/run/GHOST_configuration.npz')
run_details = config_frame['run_details'].item()
parameter_details = config_frame['parameter_details'].item()
network_details = config_frame['network_details'].item()

#MAKE RUN/PARAMETER/NETWORK DATA GLOBAL ACROSS ALL SUBMODULES
__builtin__.run_details = run_details
__builtin__.parameter_details = parameter_details
__builtin__.network_details = network_details

#UPDATE GLOBAL WRITE STAGE
write_stage = 'final_write'
__builtin__.write_stage = write_stage

class Work(object):
    def __init__(self):
    
        #CREATE QUEUE FOR ADDING DATA TO
        self.work = Queue.Queue()
        
        #GET TEMPORARY PROCESSED REFERENCES - GET REFERENCES FROM OUTPUT NUMPY FILES IN TEMPORARY DIRECTORY
        processed_temporary_references = crossnetwork_processing_modules.get_processed_temporary_references()
    
        #MAKE SURE HAVE SOME DATA TO PROCESS
        if len(processed_temporary_references) > 0:
    
            #TEST FOR PRESENCE OF FINAL STATION PROCESS INFORMATION DIRECTORY. USED FOR GETTING ALL VALID FULLY QUALITY CONTROLLED STATION REFERENCES 
            #IF DOES NOT EXIST CREATE IT
            final_file_process_dir = '/esarchive/obs/ghost/%s/native/%s/final_process_valid/'%(run_details['network'],parameter_details['bsc_parameter_name'])
            does_temporary_exist = os.path.isdir(final_file_process_dir)
            if does_temporary_exist == False:
                os.makedirs(final_file_process_dir)
            #DELETE ALL CURRENT FINAL STATION FILES FOR PARAMETER/NETWORK (IN CASE OF FAILED PREVIOUS RUN AND HAVE NOT BEEN CLEANED)
            for current_file in glob.glob('%s*'%(final_file_process_dir)):
                os.remove(current_file)
    
            #ADD DATA TO QUEUE
            for ref in processed_temporary_references:
                self.work.put(ref)

    def get_next(self):
        if self.work.empty():
            return None
        return self.work.get()
        
    def test_if_start_empty(self):
        if self.work.empty():
            return True
        else:
            return False
 

def do_work(work):
    crossnetwork_processing_modules.quality_control_processing(work)
    return work
    
def master(comm):
    num_procs = comm.Get_size()
    status = MPI.Status()
    
    # generate work queue
    wq = Work()
    
    #array for appending results
    results = []
    
    #test if queue is empty to start with
    if wq.test_if_start_empty():
        # if empty - Tell all the slaves to exit by sending an empty message with DIETAG
        for rank in xrange(1, num_procs):
            comm.send(0, dest=rank, tag=DIETAG)
        return

    # Seed the slaves, send one unit of work to each slave (rank)
    working_ranks = []
    for rank in xrange(1, num_procs):
        work = wq.get_next()
        if not work: break
        comm.send(work, dest=rank, tag=WORKTAG)
        working_ranks.append(rank)
    
    # Loop over getting new work requests until there is no more work to be done
    while True:
        work = wq.get_next()
        if not work: break
    
        # Receive results from a slave
        results.append(comm.recv(source=MPI.ANY_SOURCE, tag=MPI.ANY_TAG, status=status))

        # Send the slave a new work unit
        comm.send(work, dest=status.Get_source(), tag=WORKTAG)
    
    # No more work to be done, receive all outstanding results from slaves
    for rank in working_ranks:  
        results.append(comm.recv(source=MPI.ANY_SOURCE, tag=MPI.ANY_TAG, status=status))

    # Tell all the slaves to exit by sending an empty message with DIETAG
    for rank in xrange(1, num_procs):
        comm.send(0, dest=rank, tag=DIETAG)
        
    return results


def slave(comm):
    my_rank = comm.Get_rank()
    status = MPI.Status()

    while True:
        # Receive a message from the master (the work to do)
        work = comm.recv(source=0, tag=MPI.ANY_TAG, status=status)
        # Check the tag of the received message
        if status.Get_tag() == DIETAG: break 

        # Do the work and send it back
        result = do_work(work)
        comm.send(result, dest=0)
        
def main():
    comm = MPI.COMM_WORLD
    my_rank = comm.Get_rank()
    my_name = MPI.Get_processor_name()
    
    if my_rank == 0:
        results = master(comm)
        if results is not None:
            print 'N4 DONE'
        else:
            print 'N4 NO DATA'
    else:
        slave(comm)


if __name__ == '__main__':
    main()